#! /usr/bin/env python
# -*- coding: utf-8 -*-

# Program to compute and compare the absolute humidity outside
# and in the cellar.
# The results are signaled by green and red leds:
# Green: Open window.
# Red: Close window.
# Both red and green: Difference too small to decide.
# Both leds off: Measurements are outdated.

import os
import math
import time

# Constants taken from https://en.wikipedia.org/wiki/Dew_point
# These valuations provide a maximum error of 0.1%, for
# -30°C ≤ T ≤ +35°C;
# 1% < RH < 100%;
a = 6.112
b = 17.67
c = 243.5

# Compute saturated water vapor pressure in hPa
# Param t - temperature in °C
def svp(t):
  svp = a * math.exp((b*t)/(c+t))
  return svp

# Compute actual water vapor pressure in hPa
# Param rh - relative humidity in %
# Param t - temperature in °C
def vp(rh, t):
  vp = rh/100. * svp(t)
  return vp

# Compute the absolute humidity in g/m³
# Param rh - relative humidity in %
# Param t - temperature in °C
def ah(rh, t):
  mw = 18.016 # kg/kmol (Molekulargewicht des Wasserdampfes)
  rs = 8314.3 # J/(kmol*K) (universelle Gaskonstante)
  ah = 10**5 * mw/rs * vp(rh, t)/(t + 273.15)
  return ah

# Control the green led
# Param value - Whether to turn the led on of off
def led_green(value):
  f = open('/sys/class/gpio/gpio11/value', 'w')
  f.write('1') if value else f.write('0')
  f.close()

# Control the red led
# Param value - Whether to turn the led on of off
def led_red(value):
  f = open('/sys/class/gpio/gpio9/value', 'w')
  f.write('1') if value else f.write('0')
  f.close()

# Read the actual weather data using rrdtool
# Return (header, values)
def read_weather_data():
  handle = os.popen('rrdtool lastupdate weather.rrd')
  header = handle.readline().split()
  handle.readline() # empty line
  values = handle.readline().split()
  handle.close()
  return (header, values)

def main():
  (header, values) = read_weather_data()
  # timestamp is first value
  timestamp = float(values.pop(0).rstrip(': '))
  # create dictionary {sensor name -> sensor value}
  valmap = dict(zip(header, values))
  # compute abs. humdity in cellar (sensor s5)
  ah_cellar = ah(float(valmap['hums5']), float(valmap['temps5']))
  # compute abs. humdity outside (sensor s9)
  t_out = float(valmap['temps9'])
  ah_out = ah(float(valmap['hums9']), t_out)

  # Make decisions based on comparision of abs. humdity values
  print(timestamp, ah_cellar, ah_out)
  if time.time() - timestamp > 15 * 60:
    print('Values too old')
    led_green(False)
    led_red(False)
  elif t_out < 3.0:
    print('Too cold outside - better close the window!')
    led_green(False)
    led_red(True)
  elif math.fabs(ah_out - ah_cellar) < 1:
    print('Difference too small')
    led_green(True)
    led_red(True)
  elif ah_out > ah_cellar:
    print('Close window!')
    led_green(False)
    led_red(True)
  else:
    print('Open window')
    led_green(True)
    led_red(False)

if __name__ == '__main__':
  main()
